#include "epgsearchsetup.h"
#include "epgsearchcfg.h"

const char *RedKeyMode[2];

cMenuEPGSearchSetup::cMenuEPGSearchSetup(void)
{
  RedKeyMode[0] = tr("Standard");
  RedKeyMode[1] = tr("Commands");
  Setup();
}

void cMenuEPGSearchSetup::Setup(void)
{
  data = EPGSearchConfig;
  Set();
}

void cMenuEPGSearchSetup::Set()
{
  int current = Current();
  Clear();
  Add(new cMenuEditBoolItem(  tr("Hide Mainmenu Entry"),         &data.hidemenu,       tr("no"),      tr("yes")));
  Add(new cMenuEditStraItem(tr("Red key"), &data.redkeymode, 2, RedKeyMode));
  Add(new cMenuEditBoolItem(  tr("Show progress in 'Now'"),         &data.showProgress,       tr("no"),      tr("yes")));
  if (data.showProgress)
      Add(new cMenuEditIntItem(tr("  Display Width"), &data.ProgressWidth, 4, 12));
  char* szUseUserTime = NULL;
  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 1);
  Add(new cMenuEditBoolItem(szUseUserTime, &data.ShowModes[showUserMode1].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode1].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode1].szDescription, sizeof(data.ShowModes[showUserMode1].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode1].iTime));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 2);
  Add(new cMenuEditBoolItem( szUseUserTime, &data.ShowModes[showUserMode2].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode2].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode2].szDescription, sizeof(data.ShowModes[showUserMode2].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode2].iTime));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 3);
  Add(new cMenuEditBoolItem(szUseUserTime, &data.ShowModes[showUserMode3].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode3].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode3].szDescription, sizeof(data.ShowModes[showUserMode3].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode3].iTime));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 4);
  Add(new cMenuEditBoolItem(szUseUserTime, &data.ShowModes[showUserMode4].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode4].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode4].szDescription, sizeof(data.ShowModes[showUserMode4].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode4].iTime));
  } 
  free(szUseUserTime);

 SetCurrent(Get(current));
  Display();
}

void cMenuEPGSearchSetup::Store(void)
{
    EPGSearchConfig = data;
    SetupStore("HideMenu",  EPGSearchConfig.hidemenu);
    SetupStore("RedKeyMode",  EPGSearchConfig.redkeymode);
    SetupStore("ShowProgress",  EPGSearchConfig.showProgress);
    SetupStore("ProgressWidth",  EPGSearchConfig.ProgressWidth);
    
    SetupStore("UserMode1UseIt",  EPGSearchConfig.ShowModes[showUserMode1].GetUsage());
    SetupStore("UserMode1Description",  EPGSearchConfig.ShowModes[showUserMode1].GetDescription());
    SetupStore("UserMode1Time",  EPGSearchConfig.ShowModes[showUserMode1].GetTime());
    
    SetupStore("UserMode2UseIt",  EPGSearchConfig.ShowModes[showUserMode2].GetUsage());
    SetupStore("UserMode2Description",  EPGSearchConfig.ShowModes[showUserMode2].GetDescription());
    SetupStore("UserMode2Time",  EPGSearchConfig.ShowModes[showUserMode2].GetTime());
  
    SetupStore("UserMode3UseIt",  EPGSearchConfig.ShowModes[showUserMode3].GetUsage());
    SetupStore("UserMode3Description",  EPGSearchConfig.ShowModes[showUserMode3].GetDescription());
    SetupStore("UserMode3Time",  EPGSearchConfig.ShowModes[showUserMode3].GetTime());
    
    SetupStore("UserMode4UseIt",  EPGSearchConfig.ShowModes[showUserMode4].GetUsage());
    SetupStore("UserMode4Description",  EPGSearchConfig.ShowModes[showUserMode4].GetDescription());
    SetupStore("UserMode4Time",  EPGSearchConfig.ShowModes[showUserMode4].GetTime());
    
    isyslog("EPGSearch setup stored");
}

eOSState cMenuEPGSearchSetup::ProcessKey(eKeys Key)
{
    int iTemp_useUserTime1 = data.ShowModes[showUserMode1].GetUsage();
    int iTemp_useUserTime2 = data.ShowModes[showUserMode2].GetUsage();
    int iTemp_useUserTime3 = data.ShowModes[showUserMode3].GetUsage();
    int iTemp_useUserTime4 = data.ShowModes[showUserMode4].GetUsage();

    eOSState state = cMenuSetupPage::ProcessKey(Key);

    if (iTemp_useUserTime1 != data.ShowModes[showUserMode1].GetUsage() ||
	iTemp_useUserTime2 != data.ShowModes[showUserMode2].GetUsage() ||
	iTemp_useUserTime3 != data.ShowModes[showUserMode3].GetUsage() ||
	iTemp_useUserTime4 != data.ShowModes[showUserMode4].GetUsage())
    {
	Set();
	Display();
    }
    return state;
}
