/*
 * menu_whatson.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#include "menu_whatson.h"
#include "epgsearchext.h"
#include "menu_event.h"
#include "menu_edittimer.h"
#include "menu_searchresults.h"
#include "menu_commands.h"
#include "epgsearchcfg.h"
#include <math.h>

#define HOURS(x) ((x)/100)
#define MINUTES(x) ((x)%100)

#define CHNUMWIDTH  (numdigits(Channels.MaxNumber()) + 1)

// --- cMenuWhatsOnSearchItem ------------------------------------------------------

cMenuWhatsOnSearchItem::cMenuWhatsOnSearchItem(const cEvent *Event, cChannel *Channel, bool bProgress)
{
  event = Event;
  channel = Channel;
  char *buffer = NULL;
  int TimerMatch;
  char t = Timers.GetMatch(Event, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
  char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
  char r = event->IsRunning() ? '*' : ' ';
  if (bProgress)
  {
      int progress = (int)roundf( (float)(time(NULL) - event->StartTime()) / (float)(event->Duration()) * 8.0 );
      if (progress < 0) progress = 0;
      else if (progress > 8) progress = 8;

      char szProgress[9]="";
      memset(szProgress, 0, sizeof(szProgress));
      memset(szProgress, ' ', sizeof(szProgress)-1);
      for(int i=0;i<progress;i++) szProgress[i]='|';

      asprintf(&buffer, "%d\t%.*s\t%s\t[%s]\t%c%c%c \t%s", channel->Number(), 6, channel->Name(), event->GetTimeString(), szProgress, t, v, r, event->Title());
  }
  else
      asprintf(&buffer, "%d\t%.*s\t%s\t%c%c%c \t%s", channel->Number(), 6, channel->Name(), event->GetTimeString(), t, v, r, event->Title());
  SetText(buffer, false);
}


// --- cMenuWhatsOnSearch ----------------------------------------------------------

int cMenuWhatsOnSearch::currentChannel = 0;
showMode cMenuWhatsOnSearch::currentShowMode = showNow;
const cEvent *cMenuWhatsOnSearch::scheduleEvent = NULL;
extern const char *ShowModes[];

cMenuWhatsOnSearch::cMenuWhatsOnSearch(const cSchedules *Schedules, int CurrentChannelNr)
:cOsdMenu(EPGSearchConfig.ShowModes[currentShowMode].GetDescription(), CHNUMWIDTH, 8, 6, EPGSearchConfig.ProgressWidth, 3)
{
  for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
      if (!Channel->GroupSep()) {
         const cSchedule *Schedule = Schedules->GetSchedule(Channel->GetChannelID());

         if (Schedule) {
	     const cEvent *Event = NULL;
	     switch(currentShowMode)
	     {
		 default:
		 case showNow: 
		     Event = Schedule->GetPresentEvent();
		     break;
		 case showNext:
		     Event = Schedule->GetFollowingEvent();
		     break;
		 case showUserMode1:
		 case showUserMode2:
		 case showUserMode3:
		 case showUserMode4:
		     time_t SeekTime = GetTimeT(EPGSearchConfig.ShowModes[currentShowMode].GetTime());
		     Event = Schedule->GetEventAround(SeekTime);
		     break;
	     }
	     if (Event)
               Add(new cMenuWhatsOnSearchItem(Event, Channel, (EPGSearchConfig.showProgress && currentShowMode==showNow)), Channel->Number() == CurrentChannelNr);
            }
         }
      }
  currentChannel = CurrentChannelNr;

  showMode nextShowMode = GetNextMode();
  char* szButtonGreen = EPGSearchConfig.ShowModes[nextShowMode].GetDescription();
  SetHelp((EPGSearchConfig.redkeymode==0?tr("Record"):tr("Button$Commands")), szButtonGreen, tr("Button$Schedule"), tr("Button$Search"));

  char* szTitle = NULL;
  asprintf(&szTitle, "%s - %s", tr("Overview"), EPGSearchConfig.ShowModes[currentShowMode].GetDescription());
  SetTitle(szTitle);
  free(szTitle);
  currentShowMode = nextShowMode;
}

time_t cMenuWhatsOnSearch::GetTimeT(int iTime)
{
    struct tm tm_r;
    time_t t = time(NULL);
    tm* tmnow = localtime_r(&t, &tm_r);
    tmnow->tm_hour = HOURS(iTime);
    tmnow->tm_min = MINUTES(iTime);
    return mktime(tmnow);
}

showMode cMenuWhatsOnSearch::GetNextMode()
{
    showMode nextShowMode = currentShowMode;
    time_t now = time(NULL);
    do
    {
	nextShowMode = (showMode)((nextShowMode+1) % showModeMax);
	if (nextShowMode >= showUserMode1)
	{
	    if (!EPGSearchConfig.ShowModes[nextShowMode].GetUsage())
	    {
		isyslog("EPGSearch Mode: %d (not used)", (int)nextShowMode);
		continue;
	    }

	    time_t TestTime = GetTimeT(EPGSearchConfig.ShowModes[nextShowMode].GetTime());    
	    if (TestTime < now)
	    {
		isyslog("EPGSearch Mode: %d (past)", (int)nextShowMode);
		continue;
	    }
	    break;
	}
	else
	    break;
    }
    while(true);
    isyslog("EPGSearch Mode: %d", (int)nextShowMode);
    return nextShowMode;
}
const cEvent *cMenuWhatsOnSearch::ScheduleEvent(void)
{
  const cEvent *ei = scheduleEvent;
  scheduleEvent = NULL;
  return ei;
}


eOSState cMenuWhatsOnSearch::Switch(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->event->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Record(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->event);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Commands(eKeys Key)
{
  if (HasSubMenu() || Count() == 0)
     return osContinue;
  cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
  if (mi) {
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->event));
        if (Key != kNone)
           state = menu->ProcessKey(Key);
        return state;
     }
  return osContinue;
}

eOSState cMenuWhatsOnSearch::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kRecord:
       case kRed:
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
      		if (Count())
		{
			if (EPGSearchConfig.redkeymode==0)
				state = Record();
			else
			{
				cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
	                        if (mi) {
        	                  if (mi->event) {
				     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
					}
				}
			}
		}
		break;
       case k0:
      		if (Count())
		{
			cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                        if (mi) 
                        {
       	                  if (mi->event) 
			     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
			}
		}
		break;
       case k1...k9: return Commands(Key);
       case kYellow: 
	   state = osBack;
	   currentShowMode = showNow;
                     // continue with kGreen
       case kGreen:  {
                       cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                       if (mi) {
                          scheduleEvent = mi->event;
                          currentChannel = mi->channel->Number();
                          }
                     }
                     break;
       case kBlue:   if (HasSubMenu())
			return Switch();
		     return ExtendedSearch();
		     break;
       case kOk:     if (Count())
		     {
			const cEvent *ei = ((cMenuWhatsOnSearchItem *)Get(Current()))->event;
     			if (ei) 
			{
                           cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
			   if (channel)
                    		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel()));
			}
		     }	
                     break;
       default:      break;
       }
     }
  return state;
}


